system_template = """
You will refine the given JSON and return exactly with the same keys according to the given feedback.
You will only change "system_template" and/or "template" depending on the feedback.
In the templates, you are supposed to put strings in curly braces only in the "Inputs" list
"""

human_template = """
Original JSON:
{templates}

Inputs:{inputs}

Feedback: {feedback}

Refined JSON:
"""



system_template = """
You are helpful assistant that can read code then give feedback.
This code is a streamlit implementation of a given instruction.
The key points that you need to consider are as follows:
1 - Are all the external functions imported correctly?
2 - Are each if statement has else?
3 - Is there any state management issue in the code?
4 - Is there any button to trigger the functioanlity of the code, which is expected?
"""

human_template = """
Instruction:{instruction}
Code:{code}
Feedback:
"""



system_template = f"""
You are a helpful assistant that can determine all the inputs the system should take from the user to accomplish the given instruction.
This list includes both the initial and intermediate inputs that the system should take to work properly
If the system is chat-based then, it should have an input corresponding to that as one of the system inputs.
It should be a valid Python list
"""

human_template = """
Instruction: Application that can analyze the user
System Inputs: ["answer"]

Instruction: Create a system that can summarize a powerpoint file
System Inputs:["powerpoint_file"]

Instruction: Generate a system that enable me to give the teacher field then make a chat with the teacher.
System Inputs:["message", "teacher_field"]

Instruction: Create a translator which translates to any language
System Inputs:["output_language", "source_text"]

Instruction: Create an app that I can chat with
System Inputs:["message"]

Instruction: Generate a system that can generate tweet from hashtags and give a score for the tweet.
System Inputs:["hashtags"]

Instruction: Generate a chat-based system that can analyze the given csv file.
System Inputs:["message", "csv_file"]

Instruction: Summarize a text taken from the user
System Inputs:["text"]

Instruction: Create a platform which lets the user select a lecture and then show topics for that lecture 
then give a question to the user. After user gives his/her answer, it gives a score for the answer and give explanation.
System Inputs:["lecture", "topic", "user_answer"]

Instruction: Create a system that can generate blog post related to a website
System Inputs: ["url"]

Instruction: {instruction}
System Inputs:
"""



system_template = """
As a proficient Python developer, follow the guidelines below:
1. Build upon the given "Previous Code".
2. Ensure to import all required libraries. Do NOT use pandas.compat.StringIO as it's deprecated.
3. Implement the specified function.
4. Check if the arguments are valid (not None and/or non empty)
5. If the arguments are valid, invoke the function using the provided arguments, and store the result in the indicated variable. Otherwise, assign an empty string to the indicatd variable 
5. Ensure the resultant code is error-free and fits naturally as a continuation of the "Previous Code".


It should be the complete version of this:
================================================================
# all library imports
def {function_name}({argument}):
    # complete the function and return the result
    
if {argument} is not None and len({argument}) > 0:
    {variable} = {function_name}({argument})
else:
    {variable} = ''
================================================================
Import all the Python libraries you use in the result.
Your generated function cannot be empty and should be functional. So, you cannot generate a function having only comments.
    
Let's get coding!
"""

human_template = """
Instruction: {instruction}
Function Name: {function_name}
Arguments: {argument}
Assigned Variable: {variable}
Previous Code:
{code_snippets}
Python Code Continuation:
"""



system_template = """
You are an AI agent that is good at writing how to use markdown which includes the steps of applications that the user needs to know.
Your task is by looking at the provided code, generating concise "how to use" markdown. 

Aware that you continue on this below. This lines are mandatory:
'''
# How to use

1. Enter your [OpenAI API key](https://platform.openai.com/account/api-keys) above🔑

'''

Since OpenAI API Key is mentioned once, don't mention again, try to be as concise as possible.
Don't generate redundant steps.
Start with # How to use
Then 1. Enter your [OpenAI API key](https://platform.openai.com/account/api-keys) above🔑
Then continue 2....
"""

human_template = """
App Code:{code_snippets}
"How to" Markdown:

"""


import json
import os
import re
import autopep8
from time import sleep

from langchain.chains import LLMChain
from langchain.chat_models import ChatOpenAI
from langchain.prompts.chat import (ChatPromptTemplate,
                                    HumanMessagePromptTemplate,
                                    SystemMessagePromptTemplate)

from demogpt import utils
from demogpt.chains.task_definitions import getPlanGenHelper, getTasks
from demogpt.controllers import validate

from . import prompts


class Chains:
    @classmethod
    def setLlm(
        cls,
        model,
        openai_api_key=os.getenv("OPENAI_API_KEY", ""),
        temperature=0.0,
        openai_api_base=None,
        has_gpt4=False
    ):
        cls.openai_api_key=openai_api_key
        cls.temperature=temperature
        cls.openai_api_base=openai_api_base
        cls.has_gpt4=has_gpt4
        cls.llm = ChatOpenAI(
            model=model,
            openai_api_key=openai_api_key,
            temperature=temperature,
            openai_api_base=openai_api_base,
            request_timeout=80,  max_retries=10
        )
        cls.model = model
    
    @classmethod
    def getModel(cls, change=False, temperature=0, change_model="gpt-4-0613"):
        if change and cls.has_gpt4:
            return ChatOpenAI(
                model=change_model,
                openai_api_key=cls.openai_api_key,
                temperature=temperature,
                openai_api_base=cls.openai_api_base,
                request_timeout=80,  max_retries=10
        )
        
        if temperature > 0:
            return ChatOpenAI(
                model=cls.model,
                openai_api_key=cls.openai_api_key,
                temperature=temperature,
                openai_api_base=cls.openai_api_base,
                request_timeout=80,  max_retries=10
        )
        
        return cls.llm
        
    @classmethod
    def setModel(cls, model):
        cls.model = model

    @classmethod
    def getChain(cls, system_template="", human_template="", change=False, change_model="gpt-4-0613", temperature=0, **kwargs):
        prompts = []
        if system_template:
            prompts.append(SystemMessagePromptTemplate.from_template(system_template))
        if human_template:
            prompts.append(HumanMessagePromptTemplate.from_template(human_template))
        chat_prompt = ChatPromptTemplate.from_messages(prompts)
        return LLMChain(llm=cls.getModel(change=change, temperature=temperature, change_model=change_model), prompt=chat_prompt).run(**kwargs)
    
    @classmethod
    def title(cls, instruction):
        return cls.getChain(
            system_template=prompts.title.system_template,
            human_template=prompts.title.human_template,
            change=False,
            temperature=0.8,
            instruction=instruction
        ).replace('"','').replace("'","")

    @classmethod
    def appType(cls, instruction):
        app_type = cls.getChain(
            system_template=prompts.app_type.system_template,
            human_template=prompts.app_type.human_template,
            change=True,
            instruction=instruction,
        )

        return json.loads(app_type)

    @classmethod
    def systemInputs(cls, instruction):
        return cls.getChain(
            system_template=prompts.system_inputs.system_template,
            human_template=prompts.system_inputs.human_template,
            change=False,
            instruction=instruction,
        )

    @classmethod
    def planWithInputs(cls, instruction, system_inputs, app_type):
        TASK_DESCRIPTIONS, TASK_NAMES, TASK_DTYPES = getTasks(app_type)[:3]
        helper = getPlanGenHelper(app_type)
        plan = cls.getChain(
            system_template=prompts.plan_with_inputs.system_template,
            human_template=prompts.plan_with_inputs.human_template,
            change=False,
            instruction=instruction,
            system_inputs=system_inputs,
            helper=helper,
            TASK_DESCRIPTIONS=TASK_DESCRIPTIONS,
            TASK_NAMES=TASK_NAMES,
            TASK_DTYPES=TASK_DTYPES,
        )
        return cls.refinePlan(plan)

    @classmethod
    def planFeedback(cls, instruction, plan):
        feedback = cls.getChain(
            system_template=prompts.plan_feedback.system_template,
            human_template=prompts.plan_feedback.human_template,
            change=False,
            instruction=instruction,
            plan=plan,
        )

        return json.loads(feedback)

    @classmethod
    def planRefiner(cls, instruction, plan, feedback, app_type):
        _, TASK_NAMES, _, TASK_PURPOSES = getTasks(app_type)[:4]
        return cls.getChain(
            system_template=prompts.plan_refiner.system_template,
            human_template=prompts.plan_refiner.human_template,
            change=True,
            instruction=instruction,
            plan=plan,
            feedback=feedback,
            TASK_NAMES=TASK_NAMES,
            TASK_PURPOSES=TASK_PURPOSES,
        )

    @classmethod
    def tasks(cls, instruction, plan, app_type):
        TASK_DESCRIPTIONS, TASK_NAMES= getTasks(app_type)[:2]
        
        task_list = cls.getChain(
            system_template=prompts.tasks.system_template,
            human_template=prompts.tasks.human_template,
            instruction=instruction,
            plan=plan,
            TASK_DESCRIPTIONS=TASK_DESCRIPTIONS,
            TASK_NAMES=TASK_NAMES,
        )
        try:
            tasks = json.loads(task_list)
        except:
            print("Switching to 16k...")
            task_list = cls.getChain(
                system_template=prompts.tasks.system_template,
                human_template=prompts.tasks.human_template,
                change=True,
                change_model="gpt-3.5-turbo-16k-0613",
                instruction=instruction,
                plan=plan,
                TASK_DESCRIPTIONS=TASK_DESCRIPTIONS,
                TASK_NAMES=TASK_NAMES,
            )
            tasks = json.loads(task_list)
            
        return utils.reformatTasks(tasks)

    @classmethod
    def taskController(cls, tasks, app_type):
        return validate(tasks, app_type)

    @classmethod
    def planController(cls, plan, app_type):
        return validate(plan, app_type)

    @classmethod
    def refineTasks(cls, instruction, tasks, feedback, app_type):
        _, TASK_NAMES, _, TASK_PURPOSES = getTasks(app_type)[:4]
        
        try:
            task_list = cls.getChain(
                system_template=prompts.task_refiner.system_template,
                human_template=prompts.task_refiner.human_template,
                instruction=instruction,
                tasks=tasks,
                feedback=feedback,
                TASK_NAMES=TASK_NAMES,
                TASK_PURPOSES=TASK_PURPOSES
                )
            tasks = json.loads(task_list)
        except:
            print("Switching to 16k...")
            task_list = cls.getChain(
                system_template=prompts.task_refiner.system_template,
                human_template=prompts.task_refiner.human_template,
                change=True,
                change_model="gpt-3.5-turbo-16k-0613",
                instruction=instruction,
                tasks=tasks,
                feedback=feedback,
                TASK_NAMES=TASK_NAMES,
                TASK_PURPOSES=TASK_PURPOSES,
            )
            tasks = json.loads(task_list)
            
        return utils.reformatTasks(tasks)

    @classmethod
    def combine(cls, instruction, code_snippets, plan):
        code = cls.getChain(
            system_template=prompts.combine.system_template,
            human_template=prompts.combine.human_template,
            instruction=instruction,
            code_snippets=code_snippets,
            plan=plan,
        )
        return utils.refine(code)
    
    @classmethod
    def howToUse(cls, code_snippets):
        steps = cls.getChain(
            system_template=prompts.how_to_use.system_template,
            human_template=prompts.how_to_use.human_template,
            code_snippets=code_snippets
        )
        
        total_code = f'st.sidebar.markdown("""{steps}""")\n'
        return total_code
    
    @classmethod
    def about(cls, instruction, title):
        markdown = cls.getChain(
            system_template=prompts.about.system_template,
            human_template=prompts.about.human_template,
            instruction=instruction,
            title=title
        )
        
        code = f'\nst.sidebar.markdown("# About")\nst.sidebar.markdown("{markdown}")'
        return code

    @classmethod
    def imports(cls, code_snippets):
        code = cls.getChain(
            system_template=prompts.imports.system_template,
            human_template=prompts.imports.human_template,
            code_snippets=code_snippets,
        )
        return utils.refine(code)

    @classmethod
    def combine_v2(cls, code_snippets, function_names):
        code = cls.getChain(
            system_template=prompts.combine_v2.system_template,
            human_template=prompts.combine_v2.human_template,
            change=True,
            change_model="gpt-3.5-turbo",
            code_snippets=code_snippets,
            function_names=function_names,
        )
        code = utils.refine(code)
        code = autopep8.fix_code(code)
        
        has_problem = utils.catchErrors(code)
        
        if has_problem:
            print("Switching to the 16k...")
            code = cls.getChain(
                system_template=prompts.combine_v2.system_template,
                human_template=prompts.combine_v2.human_template,
                change=True,
                change_model="gpt-3.5-turbo-16k-0613",
                code_snippets=code_snippets,
                function_names=function_names
            )
            code = utils.refine(code)
        
        return code

    @classmethod
    def feedback(cls, instruction, code):
        return cls.getChain(
            system_template=prompts.feedback.system_template,
            human_template=prompts.feedback.human_template,
            instruction=instruction,
            code=code,
        )

    @classmethod
    def refinePlan(cls, plan):
        pattern = r"\[[a-zA-Z0-9_]+\(.*\)"
        steps = plan.strip().split("\n")
        refined_plan = []
        index = 1
        for i in range(len(steps)):
            step = steps[i]
            # If current step contains the pattern or next step contains the pattern, then retain
            if re.search(pattern, step):
                # Remove existing numbering
                current_step = re.sub(r"^\d+\.", "", step).strip()
                refined_plan.append(f"{index}. {current_step}")
                index += 1
        return "\n".join(refined_plan)
    
    @classmethod
    def addAboutAndHTU(cls, instruction, title, code_snippets):
        sleep(1)
        how_to_markdown = cls.howToUse(code_snippets=code_snippets)
        sleep(2)
        about = cls.about(instruction=instruction, title=title)
        pattern = r'(openai_api_key\s*=\s*st\.sidebar\.text_input\((?:[^()]*|\([^)]*\))*\))'
        # replacement string with additional code
        replacement = how_to_markdown + r'\1' + about
        # substitute using regex
        final_code = re.sub(pattern, replacement, code_snippets, flags=re.DOTALL)
        return final_code
    
    @classmethod
    def getAboutAndHTU(cls, instruction, title, code_snippets):
        sleep(1)
        how_to = cls.howToUse(code_snippets=code_snippets)
        sleep(2)
        about = cls.about(instruction=instruction, title=title)
        return how_to, about 

    @classmethod
    def refine(cls, instruction, code, feedback):
        code = cls.getChain(
            system_template=prompts.refine.system_template,
            human_template=prompts.refine.human_template,
            instruction=instruction,
            code=code,
            feedback=feedback,
        )
        return utils.refine(code)

    @classmethod
    def final(cls, draft_code):
        code = cls.getChain(
            system_template=prompts.final.system_template,
            human_template=prompts.final.human_template,
            draft_code=draft_code,
        )
        return utils.refine(code)



FEEDBACK_PROMPT = """
You are Streamlit and Python expert that can detect the problems of the given Streamlit code designed to do the given instruction.
You are supposed to check 5 types of problems in the code:
1. All variables and functions should be used after they are defined.
2. If any variable initialized with "if" statement then it should have "else" to get rid of NameError.
3. Since Streamlit is a stateless library, all langchain functions should be called after all the parameters are taken.
4. All buttons must be form buttos. 
It is important because form button is preserving the state of the textareas under it. You can find the classical example below:
```
with st.form("my_form"):
   st.write("Inside the form")
   slider_val = st.slider("Form slider")
   checkbox_val = st.checkbox("Form checkbox")

   # Every form must have a submit button.
   submitted = st.form_submit_button("Submit")
   if submitted:
       st.write("slider", slider_val, "checkbox", checkbox_val)

st.write("Outside the form")
```
5. All the display related parts must be under form button.
################################################################
You cannot generate code, you can only analyze and give feedback for these points.

If you can find any problem related to those 5 points list them.
If you cannot find any problem in the code, only say <SUCCESS>

Instruction:{instruction}
Code:{result}
"""

REFINEMENT_PROMPT = """
You are Streamlit and Python expert that can refine the given code according to the taken feedback.
The draft code is written for the given instruction but there are somme problem in the code.
According to the given feedback, refine the code without creating a new problem or deleting parts.

Refined Code:
"""



system_template = """
You are a coding assistant specialized in working with Streamlit applications and error-free code generation. 
Your task is, paying special attention to user input handling, state management and not getting any "not defined" error because of if statements without else. 
Don't forget to add title with already defined st.title line
Generate nothing else but only the code so that it can be directly used.
"""

human_template = """
Refine the Original Code like in the following order:

Step-1 Write all the import statements from the Draft Code.

Step-2 Write all the function definitions from the Draft Code.

Step-3 Get input from the user.

Step-4 Put a submit button with an appropriate title.

Step-5 Call functions only if all user inputs are taken and the button is clicked.


Draft Code: 
{draft_code}
################################
Final Code:
"""



system_template = """
You are a Python developer that is responsible for only finding and writing all the function imports in the Python codei.
The imports could be anywhere in the code, beginning, middle, inside of functions...
Your task is writing a Python code including all the imports in the original code but nothing else
"""

human_template = """
Original Code:{code_snippets}
Imports:
"""



system_template = """
Generate a prompt to guide the model in executing specific role. It acts as directives, providing the context and structure needed for the model to respond appropriately.

Components:
1. "system_template": Describes the model's role and task for a given instruction. This string will be used with system_template.format(...) so only used curly braces for inputs
2. "human_input": It is one of the input keys from the "Inputs" list. It should be the most appropriate one that you think it is coming from chat input. 
2. "variety": Indicates how creative or deterministic the model's response should be.
3. "function_name": A unique identifier for the specific task or instruction.

IMPORTANT NOTE:
- Write "system_template" in a way that, system_template.format(input=something for input in inputs) work.
It should also have {{chat_history}}
What I mean is that, put all the elements of Inputs inside of system_template with curly braces so that I can format it with predefined parameters.
Always put the most similar variable name which should be coming from chat input in curly braces at the end .
It should be strictly a JSON format so that it can be directly used by json.loads function in Python.
"""

human_template = """
IMPORTANT NOTE:
- ONLY the variables listed under "Inputs" MUST be included in either the "system_template" section within curly braces (e.g., '{{variable_name}}'). Do NOT include any other parameters within curly braces.
- Ensure that the exact variable names listed in "Inputs" are used without any modifications.
- If a variable is listed in "Inputs," it must appear within curly braces in the "system_template".
=========================================
Instruction: Generate a blog post from a title.
Inputs: ["human_input","title"]
Args: {{
"system_template":"
You are a chatbot having a conversation with a human. You are supposed to write a blog post from given title. Human want you to generate a blog post but you are also open to feedback and according to the given feedback, you can refine the blog \n\nTitle:{{title}}\n\n{{chat_history}}\nHuman: {{human_input}}\nBlogger:",
"human_input":"human_input",
"variety": "True",
"function_name": "chat_blogger"
}}
##########################################
Instruction: Talk like a psychologist with a given tone.
Inputs: ["talk_input","tone"]
Args: {{
"system_template": "You are a psychologist. Reply to your patience with the given tone\n\nTone:{{tone}}\n\n{{chat_history}}\nPatience: {{talk_input}}\nPsychologist:",
"human_input":"talk_input",
"variety": "False",
"function_name": "talk_like_a_psychologist"
}}
##########################################
Instruction: Answer question related to the uploaded powerpoint file.
Inputs: ["question","powerpoint_doc"]
Args: {{
"system_template": "You are a chatbot having a conversation with a human.\n\nGiven the following extracted parts of a long document, chat history and a question, create a final answer.\n\n{{powerpoint_doc}}\n\n{{chat_history}}\nHuman: {{question}}\nChatbot:",
"human_input":"question",
"variety": "False",
"function_name": "talk_like_a_psychologist"
}}
##########################################
Instruction: Talk like a mathematician
Inputs: ["human_input"]
Args: {{
"system_template": "You are a mathematician. Solve the human's mathematics problem as efficient as possible.\n\n{{chat_history}}\nHuman: {{human_input}}\nMathematician:",
"human_input":"human_input",
"variety": "True",
"function_name": "solveMathProblem"
}}
##########################################
Instruction:{instruction}
Inputs:{inputs}
Args:
"""



human_template = """
variable: url
instruction: Get website URL from the user
placeholder:Enter website URL

variable: source_text
instruction: Get source text from the user
placeholder:Type the source text

variable: input_language
instruction: Get the input language from the user
placeholder:Enter the input language

variable: {variable}
instruction: {instruction}
placeholder:

"""



import os
import sys

sys.path.append(os.path.abspath("demogpt/"))
import json
import unittest

from tqdm import tqdm

from . import utils
from .chains.chains import Chains
from .chains.task_chains import TaskChains
from .test_cases import CODE_SNIPPETS, INSTRUCTIONS, TEST_CASES, TOOL_EXAMPLES


class TestDemoGPT(unittest.TestCase):
    TEST_INDEX = 5
    INSTRUCTION = INSTRUCTIONS[TEST_INDEX]
    REFINE_ITERATIONS = 10
    # "Create a system that can summarize a content taken from url then create a blog post on the summarization"
    # "Create a system that can solve any math problem"
    TITLE = "My App"

    @classmethod
    def setUpClass(cls):
        cls.f = open(f"test_{TestDemoGPT.TEST_INDEX}.log", "w")

        # it sets the model name
        model_name = "gpt-3.5-turbo-0613"

        Chains.setLlm(model_name)
        TaskChains.setLlm(model_name)

    @classmethod
    def writeToFile(cls, title, res, instruction):
        cls.f.write(title)
        cls.f.write("\n")
        cls.f.write(instruction)
        cls.f.write("\n")
        cls.f.write(res)
        cls.f.write("\n")
        cls.f.flush()

    @classmethod
    def writeFinalToFile(cls, res, instruction):
        with open(f"test_final_{TestDemoGPT.TEST_INDEX}.py", "w") as f:
            f.write("#" + instruction + "\n")
            f.write(res)
            f.flush()

    def test_plan(self):
        for test_case in tqdm(TEST_CASES):
            instruction = test_case["instruction"]
            instruction = (
                "Create a system that can summarize a website from the given URL."
            )
            plan = Chains.plan(instruction)
            self.writeToFile("PLAN", plan, instruction)
            break

    def test_tasks(self):
        for test_case in tqdm(TEST_CASES):
            instruction = test_case["instruction"]
            plan = test_case["plan"]
            task_list = Chains.tasks(instruction=instruction, plan=plan)
            self.writeToFile("TASK LIST", json.dumps(task_list, indent=4), instruction)

    def test_feedback(self):
        for test_id in range(4, 5):
            instruction = INSTRUCTIONS[test_id]
            with open(f"test_final_{test_id}.py") as f:
                code = f.read()
            feedback = Chains.feedback(instruction=instruction, code=code)
            self.writeToFile("FEEDBACK", feedback, instruction)

    def test_refine(self):
        for test_id in range(4, 5):
            instruction = INSTRUCTIONS[test_id]
            with open(f"test_final_{test_id}.py") as f:
                code = f.read()
            feedback = Chains.feedback(instruction=instruction, code=code)
            refined_code = Chains.refine(
                instruction=instruction, code=code, feedback=feedback
            )
            self.writeToFile("REFINED CODE", refined_code, instruction)

    def test_final(self):
        for test_case in tqdm(CODE_SNIPPETS):
            instruction = test_case["instruction"]
            code_snippets = test_case["code_snippets"]
            code_snippets = utils.IMPORTS_CODE_SNIPPET + code_snippets
            final_code = Chains.final(draft_code=code_snippets)
            self.writeToFile("FINAL CODE", final_code, instruction)

    def test_task_ui_input_text(self):
        for example in TOOL_EXAMPLES["ui_input_text"]:
            instruction = example["instruction"]
            variable = example["variable"]
            res = TaskChains.uiInputText(instruction=instruction, variable=variable)
            self.writeToFile("UI INPUT TEXT", res, instruction)

    def test_task_ui_output_text(self):
        for example in TOOL_EXAMPLES["ui_input_text"]:
            instruction = example["instruction"]
            args = example["args"]
            res = TaskChains.uiOutputText(instruction=instruction, args=args)
            self.writeToFile("UI OUTPUT TEXT", res, instruction)

    def test_task_prompt_template(self):
        for example in TOOL_EXAMPLES["ui_input_text"]:
            instruction = example["instruction"]
            inputs = example["inputs"]
            res = TaskChains.promptChatTemplate(instruction=instruction, inputs=inputs)
            self.writeToFile("PROMPT CHAT TEMPLATE", res, instruction)

    def test(self):
        title = TestDemoGPT.TITLE

        instruction = TestDemoGPT.INSTRUCTION

        plan = Chains.plan(instruction)

        self.writeToFile("PLAN", plan, instruction)

        task_list = Chains.tasks(instruction=instruction, plan=plan)

        self.writeToFile("TASK LIST", json.dumps(task_list, indent=4), instruction)

        task_controller_result = Chains.taskController(tasks=task_list)

        self.writeToFile(
            "TASK CONTROLLER RESULT",
            json.dumps(task_controller_result, indent=4),
            instruction,
        )
        for _ in range(TestDemoGPT.REFINE_ITERATIONS):
            if not task_controller_result["valid"]:
                task_list = Chains.refineTasks(
                    instruction=instruction,
                    tasks=task_list,
                    feedback=task_controller_result["feedback"],
                )
                self.writeToFile(
                    "REFINED TASK LIST", json.dumps(task_list, indent=4), instruction
                )
                task_controller_result = Chains.taskController(tasks=task_list)
            else:
                break

            self.writeToFile(
                "FEEDBACK", task_controller_result["feedback"], instruction
            )

        code_snippets = utils.init(title)

        self.writeToFile("CODE SNIPPETS", "", instruction)

        for task in tqdm(task_list):
            code = utils.getCodeSnippet(task, code_snippets)
            code = "#" + task["description"] + "\n" + code
            code_snippets += code
            self.writeToFile("", code, "")

        """draft_code = Chains.draft(instruction=instruction,
                                  code_snippets=code_snippets,
                                  plan=plan
                                  )
        
        self.writeToFile("COMBINED CODE",code_snippets,instruction)
        """

        final_code = Chains.final(draft_code=code_snippets)

        self.writeFinalToFile(final_code, instruction)

    def test_all(self):
        for test_case in tqdm(TEST_CASES):
            instruction = test_case["instruction"]
            TestDemoGPT.test(instruction)
        TestDemoGPT.f.close()


if __name__ == "__main__":
    TestDemoGPT.INSTRUCTION = os.environ.get("instruction", TestDemoGPT.INSTRUCTION)
    TestDemoGPT.TITLE = os.environ.get("title", TestDemoGPT.TITLE)
    unittest.test()



from . import (app_type, combine, combine_v2, feedback, final, imports, plan,
               plan_feedback, plan_refiner, plan_with_inputs,
               prompt_chat_refiner, refine, system_inputs, task_controller,
               task_refiner, tasks, title, how_to_use, about)
from .self_refinement import final_refiner
from .task_list import (chat, doc_load, doc_to_string, path_to_file,
                        prompt_template, python_coder, search, summarize,
                        ui_input_chat, ui_input_file, ui_input_text,
                        ui_output_text)



system_template = """
You are an AI agent that can generate appealing.
The app has been generated from the description. You task is generating a good title for it.
It should be attractive and increase the open rate of the app.
The title should be sarcastic.
The title cannot exceed more than 18 characters.
"""

human_template = """
Description:{instruction}
Title:
"""


system_template = """
Regenerate the code by combining all the user input parts into st.form.
It is really important not to change other parts and the final code should be error-free and cannot include placeholders, so it should be a full code.
Copy all the function definitions and library imports as is and don't modify or replace them.
Show the result when the form is submitted under the if submit_button: statement.
Keep in mind that don't miss any function definition
Don't forget to add those functions with their original definitions as is 

{function_names}

The final code content should be in the following format.
1. Get openai_api_key
2. Copy and paste all the functions as is
3. Create only a single global form
4. Under the global form, take all the user inputs
5. If form is submitted by st.form_submit_button, call the needed functions
6. Under the st.form_submit_button, show the results.

When you need to call any of the function here {function_names}, don't call any of them before st.form_submit_button
"""

human_template = """
=============================================================
DRAFT CODE 1:

openai_api_key = st.sidebar.text_input(
    "OpenAI API Key",
    placeholder="sk-...",
    value=os.getenv("OPENAI_API_KEY", ""),
    type="password",
)

def foo1():
    result = "res"
    return result

half_story = foo1()

if half_story:
    st.write(half_story)

user_choice = st.selectbox("What would you like to do next?", ["Choice1", "Choice2"])

def foo2(half_story,user_choice):
    result = half_story + user_choice
    return result

if half_story and user_choice:
    continued_story = foo2(half_story,user_choice)
else:
    continued_story = ""

if continued_story:
    st.markdown(continued_story)
#############################################################
FINAL CODE 1:

# Get openai_api_key
openai_api_key = st.sidebar.text_input(
    "OpenAI API Key",
    placeholder="sk-...",
    value=os.getenv("OPENAI_API_KEY", ""),
    type="password",
)

### Copy and paste all the functions as is

def foo1():
    result = "res"
    return result

def foo2(half_story,user_choice):
    result = half_story + user_choice
    return result
    

### Create a form

with st.form(key='story_game'):
    # Under the form, take all the user inputs
    text_input = st.text_input(label='Enter some text')
    user_choice = st.selectbox("What would you like to do next?", ["Choice1", "Choice2"])
    submit_button = st.form_submit_button(label='Submit Story')
    # If form is submitted by st.form_submit_button run the logic
    if submit_button:
        ######## Call the functions
        half_story = foo1()
        if text_input and user_choice :
            continued_story = foo2(text_input,user_choice)
        else:
            continued_story = ""
        
        ######## Show the results
        if half_story:
            #Under the st.form_submit_button, show the results.
            st.write(half_story)
        
        if continued_story:
            #Under the st.form_submit_button, show the results.
            st.markdown(continued_story)
############################################################# END OF THE CODE


=============================================================
DRAFT CODE 2:
{code_snippets}
#############################################################
FINAL CODE 2:

"""
